#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 21:00:00                  #
# ================================================== #

from ..base_plugin import BasePlugin
from .websearch import WebSearch


class Plugin(BasePlugin):
    def __init__(self):
        super(Plugin, self).__init__()
        self.id = "cmd_web_google"
        self.name = "Command: Google Web Search"
        self.description = "Allows to connect to the Web and search web pages for actual data."
        self.options = {}
        self.options["google_api_key"] = {
            "type": "text",
            "slider": False,
            "label": "Google Custom Search API KEY",
            "description": "You can obtain your own API key at https://developers.google.com/custom-search/v1/overview",
            "tooltip": "Google Custom Search API Key",
            "value": '',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["google_api_cx"] = {
            "type": "text",
            "slider": False,
            "label": "Google Custom Search CX ID",
            "description": "You will find your CX ID at https://programmablesearchengine.google.com/controlpanel/all"
                           "\nRemember to enable \"Search on ALL internet pages\" option in project settings.",
            "tooltip": "Google Custom Search CX ID",
            "value": '',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["num_pages"] = {
            "type": "int",
            "slider": False,
            "label": "Number of max pages to search per query",
            "description": "",
            "tooltip": "",
            "value": 10,
            "min": 1,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["max_page_content_length"] = {
            "type": "int",
            "slider": False,
            "label": "Max characters of page content to get (0 = unlimited)",
            "description": "",
            "tooltip": "",
            "value": 0,
            "min": 0,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["chunk_size"] = {
            "type": "int",
            "slider": False,
            "label": "Per-page content chunk size (max characters per chunk)",
            "description": "",
            "tooltip": "",
            "value": 100000,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["use_google"] = {
            "type": "bool",
            "slider": False,
            "label": "Use Google Custom Search",
            "description": "Enable Google Custom Search API (API key required)",
            "tooltip": "Google Custom Search",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        """
        self.options["use_wikipedia"] = {
            "type": "bool",
            "slider": False,
            "label": "Use Wikipedia",
            "description": "Enable above option to use Wikipedia API (free, no API key required)",
            "tooltip": "Wikipedia API",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        """
        self.options["disable_ssl"] = {
            "type": "bool",
            "slider": False,
            "label": "Disable SSL verify",
            "description": "Disables SSL verification when crawling web pages",
            "tooltip": "Disable SSL verify",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["prompt_summarize"] = {
            "type": "textarea",
            "slider": False,
            "label": "Summarize prompt",
            "description": "Prompt used for summarize web search results",
            "tooltip": "Prompt",
            "value": "Summarize in English text from website into 3 paragraphs trying to find the most important "
                     "content which can help answer for question. "
                     "If there is no info about it then return only **FAILED** without any additional info: ",
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["max_result_length"] = {
            "type": "int",
            "slider": False,
            "label": "Max result length",
            "description": "Max length of sumamrized result (characters)",
            "tooltip": "Max result length",
            "value": 1500,
            "min": 0,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["summary_max_tokens"] = {
            "type": "int",
            "slider": False,
            "label": "Max summary tokens",
            "description": "Max tokens in output when generating summary",
            "tooltip": "Max tokens",
            "value": 1500,
            "min": 0,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["summary_model"] = {
            "type": "text",
            "slider": False,
            "label": "Model used to summarize page",
            "description": "Model used to summarize page, default: gpt-3.5-turbo-1106",
            "tooltip": "Max tokens",
            "value": "gpt-3.5-turbo-1106",
            "min": 0,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.input_text = None
        self.window = None
        self.websearch = WebSearch(self)
        self.allowed_cmds = ["web_search"]
        self.order = 100

    def setup(self):
        """
        Returns available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attaches window

        :param window: Window
        """
        self.window = window

    def on_user_send(self, text):
        """Event: On user send text"""
        return text

    def on_ctx_begin(self, ctx):
        """Event: On new context begin"""
        return ctx

    def on_ctx_end(self, ctx):
        """Event: On context end"""
        return ctx

    def on_system_prompt(self, prompt):
        """Event: On prepare system prompt"""
        return prompt

    def on_ai_name(self, name):
        """Event: On set AI name"""
        return name

    def on_user_name(self, name):
        """Event: On set username"""
        return name

    def on_enable(self):
        """Event: On plugin enable"""
        pass

    def on_disable(self):
        """Event: On plugin disable"""
        pass

    def on_input_before(self, text):
        """
        Event: Before input

        :param text: Text
        """
        self.input_text = text
        return text

    def on_ctx_before(self, ctx):
        """
        Event: Before ctx

        :param ctx: Text
        """
        return ctx

    def on_ctx_after(self, ctx):
        """
        Event: After ctx

        :param ctx: ctx
        """
        return ctx

    def cmd_syntax(self, syntax):
        """Event: On cmd syntax prepare"""
        syntax += '\n"web_search": use it for search Web for more info, prepare query for search engine itself, ' \
                  'start from page 1, If you don\'t find anything or don\'t find enough information, try the next ' \
                  'page. Max pages limit: {}, params: "query", "page"'.format(self.options["num_pages"]["value"])
        return syntax

    def cmd(self, ctx, cmds):
        msg = None
        for item in cmds:
            try:
                if item["cmd"] in self.allowed_cmds:
                    if item["cmd"] == "web_search":
                        page = 1
                        if "page" in item["params"]:
                            page = int(item["params"]["page"])
                        msg = "Web search finished: '{}'".format(item["params"]["query"])
                        result, total_found, current, url = self.websearch.make_query(item["params"]["query"], page)
                        data = {
                            'content': result,
                            'url': url,
                            'page': current,
                            'total_found': total_found,
                        }
                        response = {"request": item, "result": data}
                        ctx.results.append(response)
                        ctx.reply = True
            except Exception as e:
                response = {"request": item, "result": "Error: {}".format(e)}
                ctx.results.append(response)
                ctx.reply = True
                print("Error: {}".format(e))

        if msg is not None:
            print(msg)
            self.window.statusChanged.emit(msg)
        return ctx



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #
import os

from PySide6.QtCore import Qt
from PySide6.QtGui import QFontDatabase
from PySide6.QtWidgets import QSplitter, QWidget

from .chatbox import ChatBox
from .toolbox import Toolbox
from .menu import Menu
from .dialogs import Dialogs
from .contexts import Contexts
from .attachments import Attachments


class UI:
    def __init__(self, window=None):
        """
        UI (main)

        :param window: main UI window object
        """
        self.window = window
        self.window.data = {}
        self.window.plugin_data = {}
        self.window.menus = {}
        self.window.splitters = {}
        self.window.tabs = {}
        self.window.models = {}
        self.window.path_label = {}
        self.window.config_option = {}
        self.window.plugin_option = {}

        self.chat = ChatBox(window)
        self.toolbox = Toolbox(window)
        self.contexts = Contexts(window)
        self.attachments = Attachments(window)
        self.menu = Menu(window)
        self.dialogs = Dialogs(window)

    def setup(self):
        """Setups UI"""
        # load font
        self.setup_font()

        # chat and toolbox
        self.window.chat = self.chat.setup()
        self.window.toolbox = self.toolbox.setup()

        # ctx
        self.window.ctx = QWidget()
        self.window.ctx.setLayout(self.contexts.setup())

        # set width
        self.window.ctx.setMinimumWidth(200)

        # horizontal splitter
        self.window.splitters['main'] = QSplitter(Qt.Horizontal)
        self.window.splitters['main'].addWidget(self.window.ctx)  # contexts
        self.window.splitters['main'].addWidget(self.window.chat)  # chat box
        self.window.splitters['main'].addWidget(self.window.toolbox)  # toolbox
        self.window.splitters['main'].setSizes([1, 8, 1])

        # menu
        self.menu.setup()

        # dialogs
        self.dialogs.setup()

        # set central widget
        self.window.setCentralWidget(self.window.splitters['main'])

    def setup_font(self):
        """Setups UI font"""
        path = os.path.join(self.window.config.get_root_path(), 'data', 'fonts', 'Lato', 'Lato-Regular.ttf')
        font_id = QFontDatabase.addApplicationFont(path)
        if font_id == -1:
            print("Error loading font file {}".format(path))
        else:
            family = QFontDatabase.applicationFontFamilies(font_id)[0]



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QWidget, QVBoxLayout


# video output container (with scroll
from ...utils import trans


class VideoContainer(QWidget):
    def __init__(self, window=None):
        """
        Video container with scroll

        :param window: main window
        """
        super(VideoContainer, self).__init__()
        self.window = window
        self.setStyleSheet("background-color: #000000;")

        self.label = QLabel(trans("vision.capture.label"))
        self.label.setMaximumHeight(15)
        self.label.setAlignment(Qt.AlignCenter)

        self.layout = QVBoxLayout()
        self.video = VideoLabel(window=self.window)
        self.video.setStyleSheet("background-color: #000000;")
        self.layout.addWidget(self.video)
        self.layout.addWidget(self.label)

        self.setLayout(self.layout)

    # on resize
    def resizeEvent(self, event):
        """
        On resize event

        :param event: resize event
        """
        # self.update_pos()
        super(VideoContainer, self).resizeEvent(event)


# video output label
class VideoLabel(QLabel):
    def __init__(self, text=None, window=None):
        """
        Video output label

        :param text: text
        :param window: main window
        """
        super(VideoLabel, self).__init__(text)
        self.window = window

    def mousePressEvent(self, event):
        """
        Mouse click

        :param event: mouse event
        """
        if event.button() == Qt.LeftButton:
            self.window.controller.camera.capture_frame()
            print('captured frame')
        elif event.button() == Qt.RightButton:
            pass
        elif event.button() == Qt.MiddleButton:
            pass

    def mouseDoubleClickEvent(self, event):
        """
        Mouse double click

        :param event: mouse event
        """
        if event.button() == Qt.LeftButton:
            pass
        elif event.button() == Qt.RightButton:
            pass
        elif event.button() == Qt.MiddleButton:
            pass

    def mouseReleaseEvent(self, event):
        """
        Mouse release

        :param event: mouse event
        """
        if event.button() == Qt.LeftButton:
            pass
        elif event.button() == Qt.RightButton:
            pass
        elif event.button() == Qt.MiddleButton:
            pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.09 08:00:00                  #
# ================================================== #
import os.path
import subprocess
from datetime import datetime

from ..base_plugin import BasePlugin


class Plugin(BasePlugin):
    def __init__(self):
        super(Plugin, self).__init__()
        self.id = "cmd_custom"
        self.name = "Command: Custom Commands"
        self.description = "Provides availability to create and execute custom commands"
        self.options = {}
        self.options["cmds"] = {
            "type": "dict",
            "keys": {
                "name": "text",
                "instruction": "text",
                "params": "text",
                "cmd": "text",
            },
            "slider": False,
            "label": "Your custom commands",
            "description": "Add your custom commands here, use {placeholders} to receive params, you can also use predefined "
                           "placeholders: {_time}, {_date}, {_datetime}, {_file}, {_home)",
            "tooltip": "See the documentation for more details about examples, usage and list of predefined placeholders",
            "value": [
                {
                    "name": "example_cmd",
                    "instruction": "execute tutorial test command by replacing 'hello' and 'world' params with some funny words",
                    "params": "hello, world",
                    "cmd": 'echo "Response from {hello} and {world} at {_time}"',
                },
            ],
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.window = None
        self.order = 100

    def setup(self):
        """
        Returns available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attaches window

        :param window: Window
        """
        self.window = window

    def on_user_send(self, text):
        """Event: On user send text"""
        return text

    def on_ctx_begin(self, ctx):
        """Event: On new context begin"""
        return ctx

    def on_ctx_end(self, ctx):
        """Event: On context end"""
        return ctx

    def on_system_prompt(self, prompt):
        """Event: On prepare system prompt"""
        return prompt

    def on_ai_name(self, name):
        """Event: On set AI name"""
        return name

    def on_user_name(self, name):
        """Event: On set username"""
        return name

    def on_enable(self):
        """Event: On plugin enable"""
        pass

    def on_disable(self):
        """Event: On plugin disable"""
        pass

    def on_input_before(self, text):
        """
        Event: Before input

        :param text: Text
        """
        return text

    def on_ctx_before(self, ctx):
        """
        Event: Before ctx

        :param ctx: Text
        """
        return ctx

    def on_ctx_after(self, ctx):
        """
        Event: After ctx

        :param ctx: ctx
        """
        return ctx

    def cmd_syntax(self, syntax):
        """Event: On cmd syntax prepare"""
        for item in self.options["cmds"]["value"]:
            syntax += '\n"{}": {}'.format(item["name"], item["instruction"])
            if item["params"] != "":
                params = self.extract_params(item["params"])
                if len(params) > 0:
                    syntax += ', params: "{}"'.format('", "'.join(params))
        return syntax

    def extract_params(self, text):
        """Extracts params from params string"""
        params = []
        if text is None or text == "":
            return params
        params_list = text.split(",")
        for param in params_list:
            param = param.strip()
            if param == "":
                continue
            params.append(param)
        return params

    def cmd(self, ctx, cmds):
        """
        Event: On command

        :param ctx: Context
        :param cmds: Commands requests
        """
        msg = None
        for item in cmds:
            for my_cmd in self.options["cmds"]["value"]:
                if my_cmd["name"] == item["cmd"]:
                    try:
                        # prepare command
                        command = my_cmd["cmd"]

                        # append system placeholders
                        command = command.replace("{_file}", os.path.dirname(os.path.realpath(__file__)))
                        command = command.replace("{_home}", self.window.config.path)
                        command = command.replace("{_date}", datetime.now().strftime("%Y-%m-%d"))
                        command = command.replace("{_time}", datetime.now().strftime("%H:%M:%S"))
                        command = command.replace("{_datetime}", datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

                        # append custom params to command placeholders
                        if 'params' in my_cmd and my_cmd["params"].strip() != "":
                            # append params to command placeholders
                            params_list = self.extract_params(my_cmd["params"])
                            for param in params_list:
                                if param in item["params"]:
                                    command = command.replace("{" + param + "}", item["params"][param])

                        # check if command is not empty
                        if command is None or command == "":
                            msg = "Command is empty"
                            continue

                        # execute custom command
                        msg = "Running custom command: {}".format(command)
                        print(msg)
                        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                                                   stderr=subprocess.PIPE)
                        stdout, stderr = process.communicate()
                        result = None
                        if stdout:
                            result = stdout.decode("utf-8")
                            print("Result (STDOUT): {}".format(result))
                        if stderr:
                            result = stderr.decode("utf-8")
                            print("Result (STDERR): {}".format(result))
                        if result is None:
                            result = "No result (STDOUT/STDERR empty)"
                        ctx.results.append({"request": item, "result": result})
                        ctx.reply = True  # send result message
                    except Exception as e:
                        msg = "Error: {}".format(e)
                        ctx.results.append({"request": item, "result": "Error {}".format(e)})
                        ctx.reply = True
                        print(msg)

        # update status
        if msg is not None:
            self.window.statusChanged.emit(msg)
        return ctx



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #
import os

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QVBoxLayout, QLabel, QHBoxLayout, QCheckBox, QWidget, QSplitter, QTabWidget

from .widget.textarea import ChatOutput, NotepadOutput
from .widget.filesystem import FileExplorerWidget
from .input import Input
from .highlighter import MarkdownHighlighter
from ..utils import trans


class ChatBox:
    def __init__(self, window=None):
        """
        Chatbox UI

        :param window: main window object
        """
        self.window = window
        self.input = Input(window)

    def setup(self):
        """
        Setups chatbox

        :return: QSplitter
        """
        self.window.layout_input = self.input.setup()

        self.window.data['output'] = ChatOutput(self.window)

        # notepads
        self.window.data['notepad1'] = NotepadOutput(self.window)
        self.window.data['notepad2'] = NotepadOutput(self.window)
        self.window.data['notepad3'] = NotepadOutput(self.window)
        self.window.data['notepad4'] = NotepadOutput(self.window)
        self.window.data['notepad5'] = NotepadOutput(self.window)

        path = os.path.join(self.window.config.path, 'output')
        self.window.data['output_files'] = FileExplorerWidget(self.window, path)

        # markup highlighter
        self.window.data['output_highlighter'] = MarkdownHighlighter(self.window.data['output'])

        self.window.data['chat.model'] = QLabel("")
        self.window.data['chat.model'].setAlignment(Qt.AlignRight)
        context_layout = self.setup_context()

        self.window.data['chat.label'] = QLabel(trans("chatbox.label"))
        self.window.data['chat.label'].setStyleSheet(self.window.controller.theme.get_style('text_bold'))

        self.window.data['chat.plugins'] = QLabel("")
        self.window.data['chat.plugins'].setAlignment(Qt.AlignCenter)

        header = QHBoxLayout()
        header.addWidget(self.window.data['chat.label'])
        header.addWidget(self.window.data['chat.plugins'])
        header.addWidget(self.window.data['chat.model'])

        # tabs
        self.window.tabs['output'] = QTabWidget()

        # add tabs
        self.window.tabs['output'].addTab(self.window.data['output'], trans('output.tab.chat'))
        self.window.tabs['output'].addTab(self.window.data['output_files'], trans('output.tab.files'))
        self.window.tabs['output'].addTab(self.window.data['notepad1'], trans('output.tab.notepad') + " 1")
        self.window.tabs['output'].addTab(self.window.data['notepad2'], trans('output.tab.notepad') + " 2")
        self.window.tabs['output'].addTab(self.window.data['notepad3'], trans('output.tab.notepad') + " 3")
        self.window.tabs['output'].addTab(self.window.data['notepad4'], trans('output.tab.notepad') + " 4")
        self.window.tabs['output'].addTab(self.window.data['notepad5'], trans('output.tab.notepad') + " 5")

        layout = QVBoxLayout()
        layout.addLayout(header)
        layout.addWidget(self.window.tabs['output'])
        layout.addLayout(context_layout)

        output_widget = QWidget()
        output_widget.setLayout(layout)

        input_widget = QWidget()
        input_widget.setLayout(self.window.layout_input)

        # main vertical splitter
        self.window.splitters['main.output'] = QSplitter(Qt.Vertical)
        self.window.splitters['main.output'].addWidget(output_widget)
        self.window.splitters['main.output'].addWidget(input_widget)
        self.window.splitters['main.output'].setStretchFactor(0, 4)
        self.window.splitters['main.output'].setStretchFactor(1, 1)

        return self.window.splitters['main.output']

    def setup_context(self):
        """
        Setups context

        :return: QHBoxLayout        
        """
        self.window.data['output.timestamp'] = QCheckBox(trans('output.timestamp'))
        self.window.data['output.timestamp'].stateChanged.connect(
            lambda: self.window.controller.output.toggle_timestamp(self.window.data['output.timestamp'].isChecked()))

        self.window.data['prompt.context'] = QLabel("")
        self.window.data['prompt.context'].setAlignment(Qt.AlignRight)

        layout = QHBoxLayout()
        layout.addWidget(self.window.data['output.timestamp'])
        layout.addWidget(self.window.data['prompt.context'])

        return layout



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.09 12:00:00                  #
# ================================================== #
import os.path
import shutil
import ssl
from urllib.request import Request, urlopen

from ..base_plugin import BasePlugin


class Plugin(BasePlugin):
    def __init__(self):
        super(Plugin, self).__init__()
        self.id = "cmd_files"
        self.name = "Command: Files I/O"
        self.description = "Provides commands to read and write files"
        self.options = {}
        self.options["cmd_read_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Read file",
            "description": "Allows `read_file` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_append_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Append to file",
            "description": "Allows `append_file` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_save_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Save file",
            "description": "Allows `save_file` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_delete_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Delete file",
            "description": "Allows `delete_file` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_list_files"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: List files (ls)",
            "description": "Allows `list_files` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_list_dirs"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: List directories (ls)",
            "description": "Allows `list_dirs` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_mkdir"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Directory creation (mkdir)",
            "description": "Allows `mkdir` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_download_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Downloading files",
            "description": "Allows `download_file` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_rmdir"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Removing directories",
            "description": "Allows `rmdir` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_copy_file"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Copying files",
            "description": "Allows `copy` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_copy_dir"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Copying directories (recursive)",
            "description": "Allows `copy_dir` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["cmd_move"] = {
            "type": "bool",
            "slider": False,
            "label": "Enable: Move files and directories (rename)",
            "description": "Allows `move` command execution",
            "tooltip": "",
            "value": True,
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.window = None
        self.order = 100
        self.allowed_cmds = [
            "save_file",
            "read_file",
            "append_file",
            "delete_file",
            "list_files",
            "list_dirs",
            "mkdir",
            "download_file",
            "rmdir",
            "copy_file",
            "copy_dir",
            "move",
        ]

    def setup(self):
        """
        Returns available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attaches window

        :param window: Window
        """
        self.window = window

    def on_user_send(self, text):
        """Event: On user send text"""
        return text

    def on_ctx_begin(self, ctx):
        """Event: On new context begin"""
        return ctx

    def on_ctx_end(self, ctx):
        """Event: On context end"""
        return ctx

    def on_system_prompt(self, prompt):
        """Event: On prepare system prompt"""
        return prompt

    def on_ai_name(self, name):
        """Event: On set AI name"""
        return name

    def on_user_name(self, name):
        """Event: On set username"""
        return name

    def on_enable(self):
        """Event: On plugin enable"""
        pass

    def on_disable(self):
        """Event: On plugin disable"""
        pass

    def on_input_before(self, text):
        """
        Event: Before input

        :param text: Text
        """
        return text

    def on_ctx_before(self, ctx):
        """
        Event: Before ctx

        :param ctx: Text
        """
        return ctx

    def on_ctx_after(self, ctx):
        """
        Event: After ctx

        :param ctx: ctx
        """
        return ctx

    def is_cmd_allowed(self, cmd):
        """Checks if cmd is allowed"""
        key = "cmd_" + cmd
        if key in self.options and self.options[key]["value"] is True:
            return True
        return False

    def cmd_syntax(self, syntax):
        """Event: On cmd syntax prepare"""
        if self.is_cmd_allowed("read_file"):
            syntax += '\n"read_file": read data from file, params: "filename"'
        if self.is_cmd_allowed("append_file"):
            syntax += '\n"append_file": append data to file, params: "filename", "data"'
        if self.is_cmd_allowed("save_file"):
            syntax += '\n"save_file": save data to file, params: "filename", "data"'
        if self.is_cmd_allowed("delete_file"):
            syntax += '\n"delete_file": delete file, params: "filename"'
        if self.is_cmd_allowed("list_files"):
            syntax += '\n"list_files": list files in directory, params: "path"'
        if self.is_cmd_allowed("list_dirs"):
            syntax += '\n"list_dirs": list directories in directory, params: "path"'
        if self.is_cmd_allowed("mkdir"):
            syntax += '\n"mkdir": create directory, params: "path"'
        if self.is_cmd_allowed("download_file"):
            syntax += '\n"download_file": download file, params: "src", "dst"'
        if self.is_cmd_allowed("rmdir"):
            syntax += '\n"rmdir": remove directory, params: "path"'
        if self.is_cmd_allowed("copy_file"):
            syntax += '\n"copy_file": copy file, params: "src", "dst"'
        if self.is_cmd_allowed("copy_dir"):
            syntax += '\n"copy_dir": recursive copy directory, params: "src", "dst"'
        if self.is_cmd_allowed("move"):
            syntax += '\n"move": move file or directory, params: "src", "dst"'
        return syntax

    def cmd(self, ctx, cmds):
        """
        Event: On cmd
        :param ctx: Context
        :param cmds: Commands
        :return: Context
        """
        msg = None
        for item in cmds:
            try:
                if item["cmd"] in self.allowed_cmds and self.is_cmd_allowed(item["cmd"]):
                    if item["cmd"] == "save_file":
                        try:
                            msg = "Saving file: {}".format(item["params"]['filename'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['filename'])
                            data = item["params"]['data']
                            with open(path, 'w', encoding="utf-8") as file:
                                file.write(data)
                                file.close()
                                ctx.results.append({"request": item, "result": "OK"})
                                ctx.reply = True
                                print("File saved: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "append_file" and self.is_cmd_allowed("append_file"):
                        try:
                            msg = "Appending file: {}".format(item["params"]['filename'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['filename'])
                            data = item["params"]['data']
                            with open(path, 'a', encoding="utf-8") as file:
                                file.write(data)
                                file.close()
                                ctx.results.append({"request": item, "result": "OK"})
                                ctx.reply = True
                                print("File appended: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "read_file" and self.is_cmd_allowed("read_file"):
                        try:
                            msg = "Reading file: {}".format(item["params"]['filename'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['filename'])
                            if os.path.exists(path):
                                with open(path, 'r', encoding="utf-8") as file:
                                    data = file.read()
                                    ctx.results.append({"request": item, "result": data})
                                    ctx.reply = True  # send result message
                                    file.close()
                                    print("File read: {}".format(path))
                            else:
                                ctx.results.append({"request": item, "result": "File not found"})
                                ctx.reply = True
                                print("File not found: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "delete_file" and self.is_cmd_allowed("delete_file"):
                        try:
                            msg = "Deleting file: {}".format(item["params"]['filename'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['filename'])
                            if os.path.exists(path):
                                os.remove(path)
                                ctx.results.append({"request": item, "result": "OK"})
                                ctx.reply = True
                                print("File deleted: {}".format(path))
                            else:
                                ctx.results.append({"request": item, "result": "File not found"})
                                ctx.reply = True
                                print("File not found: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "list_files" and self.is_cmd_allowed("list_files"):
                        try:
                            msg = "Listing files: {}".format(item["params"]['path'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['path'])
                            if os.path.exists(path):
                                files = os.listdir(path)
                                ctx.results.append({"request": item, "result": files})
                                ctx.reply = True
                                print("Files listed: {}".format(path))
                                print("Result: {}".format(files))
                            else:
                                ctx.results.append({"request": item, "result": "Directory not found"})
                                ctx.reply = True
                                print("Directory not found: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "list_dirs" and self.is_cmd_allowed("list_dirs"):
                        try:
                            msg = "Listing directories: {}".format(item["params"]['path'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['path'])
                            if os.path.exists(path):
                                dirs = os.listdir(path)
                                ctx.results.append({"request": item, "result": dirs})
                                ctx.reply = True
                                print("Directories listed: {}".format(path))
                                print("Result: {}".format(dirs))
                            else:
                                ctx.results.append({"request": item, "result": "Directory not found"})
                                ctx.reply = True
                                print("Directory not found: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "mkdir" and self.is_cmd_allowed("mkdir"):
                        try:
                            msg = "Creating directory: {}".format(item["params"]['path'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['path'])
                            if not os.path.exists(path):
                                os.makedirs(path)
                                ctx.results.append({"request": item, "result": "OK"})
                                ctx.reply = True
                                print("Directory created: {}".format(path))
                            else:
                                ctx.results.append({"request": item, "result": "Directory already exists"})
                                ctx.reply = True
                                print("Directory already exists: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "rmdir" and self.is_cmd_allowed("rmdir"):
                        try:
                            msg = "Deleting directory: {}".format(item["params"]['path'])
                            print(msg)
                            path = os.path.join(self.window.config.path, 'output', item["params"]['path'])
                            if os.path.exists(path):
                                shutil.rmtree(path)
                                ctx.results.append({"request": item, "result": "OK"})
                                ctx.reply = True
                                print("Directory deleted: {}".format(path))
                            else:
                                ctx.results.append({"request": item, "result": "Directory not found"})
                                ctx.reply = True
                                print("Directory not found: {}".format(path))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "download_file" and self.is_cmd_allowed("download_file"):
                        try:
                            dst = os.path.join(self.window.config.path, 'output', item["params"]['dst'])
                            msg = "Downloading file: {} into {}".format(item["params"]['src'], dst)
                            print(msg)

                            # Check if src is URL
                            if item["params"]['src'].startswith("http"):
                                src = item["params"]['src']
                                # Download file from URL
                                req = Request(
                                    url=src,
                                    headers={'User-Agent': 'Mozilla/5.0'}
                                )
                                context = ssl.create_default_context()
                                context.check_hostname = False
                                context.verify_mode = ssl.CERT_NONE
                                with urlopen(req, context=context, timeout=4) as response, open(dst, 'wb') as out_file:
                                    shutil.copyfileobj(response, out_file)
                            else:
                                # Handle local file paths
                                src = os.path.join(self.window.config.path, 'output', item["params"]['src'])

                                # Copy local file
                                with open(src, 'rb') as in_file, open(dst, 'wb') as out_file:
                                    shutil.copyfileobj(in_file, out_file)

                            # handle result
                            ctx.results.append({"request": item, "result": "OK"})
                            ctx.reply = True
                            print("File downloaded: {} into {}".format(src, dst))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "copy_file" and self.is_cmd_allowed("copy_file"):
                        try:
                            msg = "Copying file: {} into {}".format(item["params"]['src'], item["params"]['dst'])
                            print(msg)
                            dst = os.path.join(self.window.config.path, 'output', item["params"]['dst'])
                            src = os.path.join(self.window.config.path, 'output', item["params"]['src'])
                            shutil.copyfile(src, dst)
                            ctx.results.append({"request": item, "result": "OK"})
                            ctx.reply = True
                            print("File copied: {} into {}".format(src, dst))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "copy_dir" and self.is_cmd_allowed("copy_dir"):
                        try:
                            msg = "Copying directory: {} into {}".format(item["params"]['src'], item["params"]['dst'])
                            print(msg)
                            dst = os.path.join(self.window.config.path, 'output', item["params"]['dst'])
                            src = os.path.join(self.window.config.path, 'output', item["params"]['src'])
                            shutil.copytree(src, dst)
                            ctx.results.append({"request": item, "result": "OK"})
                            ctx.reply = True
                            print("Directory copied: {} into {}".format(src, dst))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
                    elif item["cmd"] == "move" and self.is_cmd_allowed("move"):
                        try:
                            msg = "Moving: {} into {}".format(item["params"]['src'], item["params"]['dst'])
                            print(msg)
                            dst = os.path.join(self.window.config.path, 'output', item["params"]['dst'])
                            src = os.path.join(self.window.config.path, 'output', item["params"]['src'])
                            shutil.move(src, dst)
                            ctx.results.append({"request": item, "result": "OK"})
                            ctx.reply = True
                            print("Moved: {} into {}".format(src, dst))
                        except Exception as e:
                            ctx.results.append({"request": item, "result": "Error {}".format(e)})
                            ctx.reply = True
                            print("Error: {}".format(e))
            except Exception as e:
                ctx.results.append({"request": item, "result": "Error {}".format(e)})
                ctx.reply = True
                print("Error: {}".format(e))

        if msg is not None:
            self.window.statusChanged.emit(msg)
        return ctx




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

import datetime
import os
import requests
from openai import OpenAI


class Image:
    DIRNAME = "img"

    def __init__(self, config, window):
        """
        DALL-E Wrapper

        :param config: Config object
        """
        self.config = config
        self.window = window
        if not self.config.initialized:
            self.config.init()

    def init(self):
        """Initialize OpenAI API key"""
        pass

    def generate(self, prompt, model="dall-e-3", num=None):
        """
        Calls DALL-E API

        :param prompt: Prompt
        :param model: Model name
        :param num: Number of variants
        :return: Images paths list
        """
        if not self.window.config.data['img_raw']:
            img_prompt = '''
            1. Apply these rules if the request is related to image generation or image description; otherwise, return the user prompt as is.
            2. Translate any non-English image prompts accurately into English
            3. Start from "A photograph of..." or "An image of...", etc. DO NOT use asking, like "Please generate...", "I want to see...", etc.
            4. Use as many details as possible to describe the image.
            5. If user only want to talk then return the user prompt as is (AND ONLY their prompt, without adding any text to it).
            '''

            # call GPT for generate best prompt
            response = self.window.gpt.quick_call(prompt, img_prompt, False, 200,
                                                  self.window.config.data['img_prompt_model'], 1.0)
            if response is not None and response != "":
                prompt = response

        print("Generating from: '{}'".format(prompt))

        client = OpenAI(
            # This is the default and can be omitted
            api_key=self.config.data["api_key"],
            organization=self.config.data["organization_key"],
        )

        response = client.images.generate(
            model=model,
            prompt=prompt,
            n=num,
            size=self.config.data["img_resolution"],
        )

        # generate and download images
        paths = []
        for i in range(num):
            url = response.data[i].url
            res = requests.get(url)
            name = self.make_safe_filename(prompt) + "-" + datetime.date.today().strftime(
                "%Y-%m-%d") + "_" + datetime.datetime.now().strftime("%H-%M-%S") + "-" + str(i + 1) + ".png"
            path = os.path.join(self.config.path, self.DIRNAME, name)
            print("Downloading... [" + str(i + 1) + " of " + str(num) + "] to: " + path)
            open(path, "wb").write(res.content)
            paths.append(path)

        return paths, prompt

    def make_safe_filename(self, name):
        """
        Makes safe filename

        :param name: Filename to make safe
        :return: Safe filename
        """

        def safe_char(c):
            if c.isalnum():
                return c
            else:
                return "_"

        return "".join(safe_char(c) for c in name).rstrip("_")[:30]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QPushButton, QHBoxLayout, QLabel, QVBoxLayout

from ..widget.settings import SettingsTextarea, SettingsInput, SettingsCheckbox, SettingsSlider
from ..widget.dialog import EditorDialog
from ...utils import trans


class Preset:
    def __init__(self, window=None):
        """
        Preset editor dialog

        :param window: main UI window object
        """
        self.window = window

    def setup(self):
        """Setups preset editor dialog"""

        id = "preset.presets"
        path = self.window.config.path

        self.window.data['preset.btn.current'] = QPushButton(trans("dialog.preset.btn.current"))
        self.window.data['preset.btn.save'] = QPushButton(trans("dialog.preset.btn.save"))
        self.window.data['preset.btn.current'].clicked.connect(
            lambda: self.window.controller.presets.from_current())
        self.window.data['preset.btn.save'].clicked.connect(
            lambda: self.window.controller.presets.save())

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.window.data['preset.btn.current'])
        bottom_layout.addWidget(self.window.data['preset.btn.save'])

        section = 'preset.editor'  # prevent autoupdate current preset

        # fields
        self.window.path_label[id] = QLabel(str(path))
        self.window.config_option['preset.prompt'] = SettingsTextarea(self.window, 'preset.prompt', False, section)
        self.window.config_option['preset.filename'] = SettingsInput(self.window, 'preset.filename', False, section)
        self.window.config_option['preset.name'] = SettingsInput(self.window, 'preset.name', False, section)
        self.window.config_option['preset.ai_name'] = SettingsInput(self.window, 'preset.ai_name', False, section)
        self.window.config_option['preset.user_name'] = SettingsInput(self.window, 'preset.user_name', False, section)
        self.window.config_option['preset.img'] = SettingsCheckbox(self.window, 'preset.img',
                                                                   trans('preset.img'), False, section)
        self.window.config_option['preset.chat'] = SettingsCheckbox(self.window, 'preset.chat', trans('preset.chat'),
                                                                    False, section)
        self.window.config_option['preset.completion'] = SettingsCheckbox(self.window, 'preset.completion',
                                                                          trans('preset.completion'), False, section)
        self.window.config_option['preset.vision'] = SettingsCheckbox(self.window, 'preset.vision',
                                                                      trans('preset.vision'), False, section)
        self.window.config_option['preset.langchain'] = SettingsCheckbox(self.window, 'preset.langchain',
                                                                      trans('preset.langchain'), False, section)
        self.window.config_option['preset.assistant'] = SettingsCheckbox(self.window, 'preset.assistant',
                                                                      trans('preset.assistant'), False, section)
        self.window.config_option['preset.temperature'] = SettingsSlider(self.window, 'preset.temperature',
                                                                         '', 0, 200,
                                                                         1, 100, True, section)

        # set max width
        max_width = 240
        self.window.config_option['preset.filename'].setMaximumWidth(max_width)
        self.window.config_option['preset.name'].setMaximumWidth(max_width)
        self.window.config_option['preset.ai_name'].setMaximumWidth(max_width)
        self.window.config_option['preset.user_name'].setMaximumWidth(max_width)

        options = {}
        options['filename'] = self.add_option('preset.filename', self.window.config_option['preset.filename'])
        options['name'] = self.add_option('preset.name', self.window.config_option['preset.name'])
        options['ai_name'] = self.add_option('preset.ai_name', self.window.config_option['preset.ai_name'])
        options['user_name'] = self.add_option('preset.user_name', self.window.config_option['preset.user_name'])
        options['chat'] = self.add_raw_option(self.window.config_option['preset.chat'])
        options['completion'] = self.add_raw_option(self.window.config_option['preset.completion'])
        options['vision'] = self.add_raw_option(self.window.config_option['preset.vision'])
        options['assistant'] = self.add_raw_option(self.window.config_option['preset.assistant'])
        options['langchain'] = self.add_raw_option(self.window.config_option['preset.langchain'])
        options['img'] = self.add_raw_option(self.window.config_option['preset.img'])
        options['temperature'] = self.add_option('preset.temperature', self.window.config_option['preset.temperature'])

        self.window.config_option['preset.prompt'].setMinimumHeight(150)

        self.window.data['preset.prompt.label'] = QLabel(trans('preset.prompt'))
        options['prompt'] = QVBoxLayout()
        options['prompt'].addWidget(self.window.data['preset.prompt.label'])
        options['prompt'].addWidget(self.window.config_option['preset.prompt'])

        rows = QVBoxLayout()
        rows.addLayout(options['filename'])
        rows.addLayout(options['name'])
        rows.addLayout(options['ai_name'])
        rows.addLayout(options['user_name'])
        rows.addLayout(options['chat'])
        rows.addLayout(options['completion'])
        rows.addLayout(options['img'])
        rows.addLayout(options['vision'])
        rows.addLayout(options['assistant'])
        rows.addLayout(options['langchain'])
        rows.addLayout(options['temperature'])
        rows.addLayout(options['prompt'])

        layout = QVBoxLayout()
        layout.addLayout(rows)
        layout.addLayout(bottom_layout)

        self.window.dialog['editor.' + id] = EditorDialog(self.window, id)
        self.window.dialog['editor.' + id].setLayout(layout)
        self.window.dialog['editor.' + id].setWindowTitle(trans('dialog.preset'))

    def add_option(self, title, option, bold=False):
        """
        Adds option

        :param title: Title
        :param option: Option
        :param bold: Bold title
        """
        label_key = title + '.label'
        self.window.data[label_key] = QLabel(trans(title))
        if bold:
            self.window.data[label_key].setStyleSheet(self.window.controller.theme.get_style('text_bold'))
        layout = QHBoxLayout()
        layout.addWidget(self.window.data[label_key])
        layout.addWidget(option)

        return layout

    def add_raw_option(self, option):
        """
        Adds raw option row

        :param option: Option
        """
        layout = QHBoxLayout()
        layout.addWidget(option)
        return layout



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

import datetime
import json
import os
import time

from .tokens import num_tokens_from_context_item
from .utils import trans


class Context:
    def __init__(self, config=None):
        """
        Context

        :param config: config object
        """
        self.config = config
        self.contexts = {}
        self.items = []
        self.current_ctx = None
        self.current_thread = None
        self.current_run = None
        self.current_status = None
        self.current_mode = None

    def load_list(self):
        """Loads contexts list from file"""
        path = os.path.join(self.config.path, 'context.json')
        try:
            if os.path.exists(path):
                with open(path, 'r', encoding="utf-8") as file:
                    data = json.load(file)
                    file.close()
                    if data == "" or data is None or 'items' not in data:
                        self.contexts = {}
                        return
                    self.contexts = data['items']
        except Exception as e:
            print(e)
            self.contexts = {}

    def load(self, name):
        """
        Loads context from file

        :param name: context name (id)
        :return: context items
        """
        path = os.path.join(self.config.path, 'context', name + '.json')
        if os.path.exists(path):
            try:
                with open(path, 'r', encoding="utf-8") as file:
                    data = self.parse(json.load(file))
                    file.close()
                    if data == "" or data is None:
                        return []
                    return data
            except Exception as e:
                print("Error while loading context: {}".format(name))
                print(e)
                return []
        else:
            return []

    def parse(self, data):
        """
        Parses context data

        :param data: context items data
        :return: context items (deserialized) as objects list
        """
        items = []
        for item in data:
            ctx = ContextItem()
            ctx.deserialize(item)
            items.append(ctx)
        return items

    def update(self):
        """
        Updates current context mode

        :return: None
        """
        self.current_mode = self.config.data['mode']
        if self.current_ctx is None:
            return
        self.contexts[self.current_ctx]['mode'] = self.current_mode
        self.dump_context(self.current_ctx)

    def create_id(self):
        """
        Creates context ID

        Format: YYYYMMDDHHMMSS.MICROSECONDS.json

        :return: context id
        """
        return datetime.datetime.now().strftime("%Y%m%d%H%M%S.%f")

    def new(self):
        """
        Creates new context

        :return: created context name (id)
        """
        name = self.create_id()  # create unique id
        self.contexts[name] = {
            'id': name,
            "name": "{}".format(trans('ctx.new.prefix')),
            "date": datetime.datetime.now().strftime("%Y-%m-%d"),
            'mode': self.config.data['mode'],
            'thread': None,
            'run': None,
            'status': None,
            'initialized': False,
        }
        self.current_ctx = name
        self.current_thread = None
        self.current_mode = self.config.data['mode']
        self.items = []
        self.dump_context(name)

        return name

    def is_ctx_initialized(self):
        """
        Checks if context is initialized (name assigned)

        :return: True if initialized, False otherwise
        """
        if self.current_ctx is None:
            return
        if self.current_ctx in self.contexts:
            if 'initialized' in self.contexts[self.current_ctx]:
                return self.contexts[self.current_ctx]['initialized']
        return True  # old versions compatibility

    def set_ctx_initialized(self):
        """
        Sets context as initialized (name assigned)
        """
        if self.current_ctx is None:
            return
        if self.current_ctx in self.contexts:
            self.contexts[self.current_ctx]['initialized'] = True
            self.dump_context(self.current_ctx)

    def append_thread(self, thread):
        """
        Appends thread to context

        :param thread: thread ID
        """
        self.current_thread = thread
        if self.current_ctx is None:
            return
        if self.current_ctx in self.contexts:
            self.contexts[self.current_ctx]['thread'] = self.current_thread
            self.dump_context(self.current_ctx)

    def append_run(self, run):
        """
        Appends run ID to context

        :param run: run ID
        """
        self.current_run = run
        if self.current_ctx is None:
            return
        if self.current_ctx in self.contexts:
            self.contexts[self.current_ctx]['run'] = self.current_run
            self.dump_context(self.current_ctx)

    def append_status(self, status):
        """
        Appends status to context

        :param status: status
        """
        self.current_status = status
        if self.current_ctx is None:
            return
        if self.current_ctx in self.contexts:
            self.contexts[self.current_ctx]['status'] = self.current_status
            self.dump_context(self.current_ctx)

    def dump_context(self, name):
        """
        Dumps context to file

        :param name: context name (id)
        """
        if not self.config.data['store_history']:
            return

        try:
            # update current context items
            items_path = os.path.join(self.config.path, 'context', name + '.json')
            items = []
            for item in self.items:
                items.append(item.serialize())
            dump = json.dumps(items, indent=4)
            with open(items_path, 'w', encoding="utf-8") as f:
                f.write(dump)
                f.close()

            # update contexts index
            index_path = os.path.join(self.config.path, 'context.json')
            data = {}
            data['items'] = self.contexts.copy()
            data['__meta__'] = self.config.append_meta()
            dump = json.dumps(data, indent=4)
            with open(index_path, 'w', encoding="utf-8") as f:
                f.write(dump)
                f.close()

        except Exception as e:
            print("Error while dumping context: {}".format(name))
            print(e)

    def get_list(self):
        """
        Gets context items sorted descending by date

        :return: contexts dict
        """
        return dict(sorted(self.contexts.items(), reverse=True))

    def get_name_by_idx(self, idx):
        """
        Gets context name (id) by index

        :param idx: index
        :return: context name (id)
        """
        i = 0
        for name in self.get_list():
            if i == idx:
                return name
            i += 1

    def get_first_ctx(self):
        """
        Returns first context from list

        :return: context name (id)
        """
        for name in self.get_list():
            return name

    def get_context_by_name(self, name):
        """
        Returns context by name

        :param name: context name (id)
        :return: context dict
        """
        if name in self.contexts:
            return self.contexts[name]

    def delete_ctx(self, name):
        """
        Deletes context by name

        :param name: context name (id)
        """
        if name in self.contexts:
            del self.contexts[name]
            path = os.path.join(self.config.path, 'context', name + '.json')
            if os.path.exists(path):
                try:
                    os.remove(path)
                except Exception as e:
                    print(e)

    def prepare(self):
        """Prepares context for prompt"""
        # if no contexts, create new one
        if len(self.contexts) == 0:
            self.new()

    def delete_all_ctx(self):
        """Deletes all contexts"""
        # delete all context files
        for name in self.contexts:
            path = os.path.join(self.config.path, 'context', name + '.json')
            if os.path.exists(path):
                try:
                    os.remove(path)
                except Exception as e:
                    print(e)
        self.contexts = {}

        # update contexts index
        index_path = os.path.join(self.config.path, 'context.json')
        data = {}
        data['items'] = {}
        data['__meta__'] = self.config.append_meta()
        try:
            dump = json.dumps(data, indent=4)
            with open(index_path, 'w', encoding="utf-8") as f:
                f.write(dump)
                f.close()
        except Exception as e:
            print(e)

        # delete all txt history files from history dir
        path = os.path.join(self.config.path, 'history')
        for file in os.listdir(path):
            if file.endswith('.txt'):
                try:
                    os.remove(os.path.join(path, file))
                except Exception as e:
                    print(e)

    def count_prompt_items(self, model, mode, used_tokens=100, max_tokens=1000):
        """
        Counts context items to add to prompt

        :param model: model
        :param mode: mode
        :param used_tokens: used tokens
        :param max_tokens: max tokens
        :return: context items count, context tokens count
        """
        i = 0
        # loop on items from end to start
        tokens = used_tokens
        context_tokens = 0
        for item in reversed(self.items):
            num = num_tokens_from_context_item(item, mode, model)  # get num tokens for input and output
            tokens += num
            if tokens > max_tokens:
                break
            context_tokens += num
            i += 1

        return i, context_tokens

    def get_prompt_items(self, model, used_tokens=100, max_tokens=1000):
        """
        Returns context items to add to prompt

        :param model: model
        :param used_tokens: used tokens
        :param max_tokens: max tokens
        :return: context items list
        """
        items = []
        # loop on items from end to start
        tokens = used_tokens
        for item in reversed(self.items):
            tokens += num_tokens_from_context_item(item, model)
            if tokens > max_tokens:
                break
            items.append(item)

        # reverse items
        items.reverse()
        return items

    def get_all_items(self):
        """
        Returns all context items
        :return: context items list
        """
        items = []
        for item in reversed(self.items):
            items.append(item)

        # reverse items
        items.reverse()
        return items

    def clear(self):
        """Clears context"""
        self.items = []

    def select(self, name):
        """
        Selects context

        :param name: context name (id)
        """
        if name in self.contexts:
            ctx = self.contexts[name]
            self.current_ctx = name
            self.current_thread = None
            if 'thread' in ctx:
                self.current_thread = ctx['thread']
            if 'mode' in ctx:
                self.current_mode = ctx['mode']

            self.items = self.load(name)

    def add(self, item):
        """
        Adds item to contexts and saves context

        :param item: item to add
        """
        self.items.append(item)  # add item to context

        # save context to file
        self.store()

    def store(self):
        """Stores current context to file"""
        if self.current_ctx is not None and self.current_ctx in self.contexts:
            self.dump_context(self.current_ctx)

    def get_total_tokens(self):
        """
        Returns current prompt tokens

        :return: total tokens
        """
        last = self.get_last()
        if last is not None:
            return last.total_tokens
        return 0

    def count(self):
        """
        Counts context items

        :return: context items count
        """
        return len(self.items)

    def all(self):
        """
        Returns context items

        :return: context items
        """
        return self.items

    def get(self, index):
        """
        Returns context item by index

        :param index: item index
        :return: context item
        """
        if index < len(self.items):
            return self.items[index]

    def get_last(self):
        """
        Returns last item from context

        :return: last context item
        """
        if len(self.items) > 0:
            return self.items[-1]
        return None

    def get_tokens_left(self, max):
        """
        Returns remaining tokens in context

        :param max: max tokens
        :return: remaining tokens in context
        """
        return max - self.get_total_tokens()

    def check(self, threshold, max_total):
        """
        Checks context and clear if limit exceeded

        :param threshold: threshold
        :param max_total: max total tokens
        """
        if self.get_tokens_left(max_total) <= threshold:
            self.remove_first()

    def remove_last(self):
        """Removes last item"""
        if len(self.items) > 0:
            self.items.pop()

    def remove_first(self):
        """Removes first item"""
        if len(self.items) > 0:
            self.items.pop(0)

    def get_last_tokens(self):
        """
        Returns last tokens count

        :return: last tokens
        """
        last = self.get_last()
        if last is not None:
            return last.total_tokens
        return 0


class ContextItem:
    def __init__(self, mode=None):
        """
        Context item

        :param mode: Mode (completion, chat, langchain, assistant)
        """
        self.stream = None
        self.results = []
        self.reply = False
        self.input = None
        self.output = None
        self.mode = mode
        self.thread = None
        self.msg_id = None
        self.run_id = None
        self.input_name = None
        self.output_name = None
        self.input_timestamp = None
        self.output_timestamp = None
        self.input_tokens = 0
        self.output_tokens = 0
        self.total_tokens = 0

    def set_input(self, input, name=None):
        """
        Sets input

        :param input: Input text (prompt)
        :param name: Input person name
        """
        self.input = input
        self.input_name = name
        self.input_timestamp = int(time.time())

    def set_output(self, output, name=None):
        """
        Sets output

        :param output: Output text
        :param name: Output person name
        """
        self.output = output
        self.output_name = name
        self.output_timestamp = int(time.time())

    def set_tokens(self, input_tokens, output_tokens):
        """
        Sets tokens usage

        :param input_tokens: prompt tokens
        :param output_tokens: output tokens
        """
        self.input_tokens = input_tokens
        self.output_tokens = output_tokens
        self.total_tokens = input_tokens + output_tokens

    def serialize(self):
        """
        Serializes item to dict

        :return: serialized item
        """
        return {
            'input': self.input,
            'output': self.output,
            'mode': self.mode,
            'thread': self.thread,
            'msg_id': self.msg_id,
            'run_id': self.run_id,
            'input_name': self.input_name,
            'output_name': self.output_name,
            'input_tokens': self.input_tokens,
            'output_tokens': self.output_tokens,
            'total_tokens': self.total_tokens,
            'input_timestamp': self.input_timestamp,
            'output_timestamp': self.output_timestamp
        }

    def deserialize(self, data):
        """Deserializes item from dict"""
        if 'input' in data:
            self.input = data['input']
        if 'output' in data:
            self.output = data['output']
        if 'mode' in data:
            self.mode = data['mode']
        if 'thread' in data:
            self.thread = data['thread']
        if 'msg_id' in data:
            self.msg_id = data['msg_id']
        if 'run_id' in data:
            self.run_id = data['run_id']
        if 'input_name' in data:
            self.input_name = data['input_name']
        if 'output_name' in data:
            self.output_name = data['output_name']
        if 'input_tokens' in data:
            self.input_tokens = data['input_tokens']
        if 'output_tokens' in data:
            self.output_tokens = data['output_tokens']
        if 'total_tokens' in data:
            self.total_tokens = data['total_tokens']
        if 'input_timestamp' in data:
            self.input_timestamp = data['input_timestamp']
        if 'output_timestamp' in data:
            self.output_timestamp = data['output_timestamp']

    def dump(self):
        """Dumps item to string"""
        return json.dumps(self.serialize())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction, QIcon, QCursor
from PySide6.QtWidgets import QTreeView, QMenu, QWidget, QVBoxLayout, QFileSystemModel

from ...utils import trans


class FileExplorerWidget(QWidget):
    def __init__(self, window, directory):
        super().__init__()

        self.window = window
        self.model = QFileSystemModel()
        self.model.setRootPath(directory)

        self.treeView = QTreeView()
        self.treeView.setModel(self.model)
        self.treeView.setRootIndex(self.model.index(directory))

        # self.treeView.setColumnHidden(1, True)
        # self.treeView.setColumnHidden(2, True)
        # self.treeView.setColumnHidden(3, True)

        layout = QVBoxLayout()
        layout.addWidget(self.treeView)
        self.setLayout(layout)

        self.treeView.setContextMenuPolicy(Qt.CustomContextMenu)
        self.treeView.customContextMenuRequested.connect(self.openContextMenu)

    def openContextMenu(self, position):
        indexes = self.treeView.selectedIndexes()
        if indexes:
            index = indexes[0]
            path = self.model.filePath(index)

            actions = {}
            actions['open_dir'] = QAction(QIcon.fromTheme("system-file-manager"), trans('action.open_dir'), self)
            actions['open_dir'].triggered.connect(
                lambda: self.action_open_dir(path))

            actions['rename'] = QAction(QIcon.fromTheme("edit-edit"), trans('action.rename'), self)
            actions['rename'].triggered.connect(
                lambda: self.action_rename(path))

            actions['delete'] = QAction(QIcon.fromTheme("edit-delete"), trans('action.delete'), self)
            actions['delete'].triggered.connect(
                lambda: self.action_delete(path))

            menu = QMenu(self)
            menu.addAction(actions['rename'])
            menu.addAction(actions['open_dir'])
            menu.addAction(actions['delete'])

            menu.exec(QCursor.pos())

    def action_open_dir(self, path):
        """
        Open dir action handler

        :param event: mouse event
        """
        self.window.controller.files.open_dir(path)

    def action_rename(self, path):
        """
        Rename action handler

        :param event: mouse event
        """
        self.window.controller.files.rename(path)

    def action_delete(self, path):
        """
        Delete action handler

        :param event: mouse event
        """
        self.window.controller.files.delete(path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

import threading

import requests
from PySide6.QtCore import QObject, Signal
from pydub import AudioSegment
from pydub.playback import play
import io

from ..base_plugin import BasePlugin


class Plugin(BasePlugin):
    def __init__(self):
        super(Plugin, self).__init__()
        self.id = "audio_azure"
        self.name = "Audio Output (MS Azure)"
        self.description = "Enables audio/voice output (speech synthesis) using Microsoft Azure API"
        self.options = {}
        self.options["azure_api_key"] = {
            "type": "text",
            "slider": False,
            "label": "Azure API Key",
            "description": "You can obtain your own API key at: https://azure.microsoft.com/en-us/services/cognitive-services/text-to-speech/",
            "tooltip": "Azure API Key",
            "value": '',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["azure_region"] = {
            "type": "text",
            "slider": False,
            "label": "Azure Region",
            "description": "Specify Azure region, e.g. eastus",
            "tooltip": "Azure Region",
            "value": 'eastus',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["voice_en"] = {
            "type": "text",
            "slider": False,
            "label": "Voice (EN)",
            "description": "Specify voice for English language, e.g. en-US-AriaNeural",
            "tooltip": "Voice (EN)",
            "value": 'en-US-AriaNeural',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["voice_pl"] = {
            "type": "text",
            "slider": False,
            "label": "Voice (PL)",
            "description": "Specify voice for Polish language, e.g. pl-PL-MarekNeural or pl-PL-AgnieszkaNeural",
            "tooltip": "Voice (PL)",
            "value": 'pl-PL-AgnieszkaNeural',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.input_text = None
        self.window = None
        self.order = 9999

    def setup(self):
        """
        Returns available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attaches window

        :param window: Window
        """
        self.window = window

    def on_user_send(self, text):
        """Event: On user send text"""
        return text

    def on_ctx_begin(self, ctx):
        """Event: On new context begin"""
        return ctx

    def on_ctx_end(self, ctx):
        """Event: On context end"""
        return ctx

    def on_system_prompt(self, prompt):
        """Event: On prepare system prompt"""
        return prompt

    def on_ai_name(self, name):
        """Event: On set AI name"""
        return name

    def on_user_name(self, name):
        """Event: On set user name"""
        return name

    def on_enable(self):
        """Event: On plugin enable"""
        pass

    def on_disable(self):
        """Event: On plugin disable"""
        pass

    def on_input_before(self, text):
        """
        Event: Before input

        :param text: Text
        :return: text
        """
        self.input_text = text
        return text

    def on_ctx_before(self, ctx):
        """
        Event: Before ctx

        :param ctx: Text
        :return: ctx
        """
        return ctx

    def on_ctx_after(self, ctx):
        """
        Event: After ctx

        :param ctx: ctx
        :return: ctx
        """
        # Check if api key is set
        if self.options['azure_api_key']['value'] is None or self.options['azure_api_key']['value'] == "":
            self.window.ui.dialogs.alert("Azure API KEY is not set. Please set it in plugins settings.")
            return ctx
        if self.options['azure_region']['value'] is None or self.options['azure_region']['value'] == "":
            self.window.ui.dialogs.alert("Azure Region is not set. Please set it in plugins settings.")
            return ctx

        text = ctx.output
        try:
            if text is not None and len(text) > 0:
                lang = self.window.config.data['lang']
                voice = None
                if lang == "pl":
                    voice = self.options['voice_pl']['value']
                elif lang == "en":
                    voice = self.options['voice_en']['value']
                tts = TTS(self.options['azure_api_key']['value'], self.options['azure_region']['value'], voice, text)
                t = threading.Thread(target=tts.run)
                t.start()
        except Exception as e:
            print(e)

        return ctx


class TTS(QObject):
    finished = Signal(object)

    def __init__(self, subscription_key, region, voice, text):
        """
        Text to speech

        :param subscription_key: Azure API Key
        :param region: Azure region
        :param voice: Voice name
        :param text: Text to speech
        """
        super().__init__()
        self.subscription_key = subscription_key
        self.region = region
        self.text = text
        self.voice = voice

    def run(self):
        """Run TTS thread"""
        url = f"https://{self.region}.tts.speech.microsoft.com/cognitiveservices/v1"
        headers = {
            "Ocp-Apim-Subscription-Key": self.subscription_key,
            "Content-Type": "application/ssml+xml",
            "X-Microsoft-OutputFormat": "audio-16khz-128kbitrate-mono-mp3"
        }
        body = f"<speak version='1.0' xmlns='http://www.w3.org/2001/10/synthesis' xml:lang='en-US'><voice name='{self.voice}'>{self.text}</voice></speak>"
        response = requests.post(url, headers=headers, data=body.encode('utf-8'))
        if response.status_code == 200:
            audio_file = response.content
            voice = AudioSegment.from_file(io.BytesIO(audio_file), format="mp3")
            play(voice)
            self.finished.emit(audio_file)
        else:
            error_msg = f"Error: {response.status_code} - {response.text}"
            print(error_msg)
            self.finished.emit(error_msg)
